@@ -1,6 +1,6 @@
-#' print.shoreline_date
+#' Print shoreline dates to console
 #'
-#' Print the dates in a `shoreline_date` object. Each date is printed
+#' Print the dates held in a `shoreline_date` object. Each date is printed
 #'  with site name, elevation and highest density region. If the isobase
 #'  direction is different or more are provided than the default, the
 #'  directions and dates associated with these are printed separately.
@@ -50,7 +50,8 @@ print.shoreline_date <- function(x, ...){
                        gsub("^[-](.*)", "\\1 BCE\n", x[[i]][[1]]$hdr_end),
                        gsub("$", " CE\n", x[[i]][[1]]$hdr_end))), sep = "")
 
-    } else if (length(x[[i]]) == 1 & x[[i]][[1]]$dispcurve_direction == 327) {
+    } else if (length(x[[i]]) == 1 & !is.na(x[[i]][[1]]$dispcurve_direction) &
+               x[[i]][[1]]$dispcurve_direction == 327) {
 
       cat("===============")
       cat("\nSite: ", x[[i]][[1]]$site_name)
@@ -64,7 +65,23 @@ print.shoreline_date <- function(x, ...){
                        gsub("^[-](.*)", "\\1 BCE\n", x[[i]][[1]]$hdr_end),
                        gsub("$", " CE\n", x[[i]][[1]]$hdr_end))), sep = "")
 
-    } else if (length(x[[i]]) == 1 & x[[i]][[1]]$dispcurve_direction != 327) {
+    } else if (length(x[[i]]) == 1 & !is.na(x[[i]][[1]]$dispcurve_direction) &
+               x[[i]][[1]]$dispcurve_direction != 327) {
+
+      cat("===============")
+      cat("\nSite: ", x[[i]][[1]]$site_name)
+      cat("\nElevation: ", x[[i]][[1]]$site_elev, "\n")
+      cat("\nIsobase direction: ", x[[i]][[1]]$dispcurve_direction, "\n")
+      cat("\n",  x[[i]][[1]]$hdr_prob*100, "% HDR:\n", sep = "")
+      cat(rbind(paste0(ifelse(grepl("^[-]", x[[i]][[1]]$hdr_start),
+                              paste(gsub("^[-]", "",
+                                         x[[i]][[1]]$hdr_start), "BCE"),
+                              gsub("$", " CE", x[[i]][[1]]$hdr_start)), "-"),
+                ifelse(grepl("^[-]", x[[i]][[1]]$hdr_end),
+                       gsub("^[-](.*)", "\\1 BCE\n", x[[i]][[1]]$hdr_end),
+                       gsub("$", " CE\n", x[[i]][[1]]$hdr_end))), sep = "")
+
+    } else if (length(x[[i]]) == 1 & is.na(x[[i]][[1]]$dispcurve_direction)) {
 
       cat("===============")
       cat("\nSite: ", x[[i]][[1]]$site_name)
